import { FlatGitignoreOptions } from 'eslint-config-flat-gitignore';
import { ParserOptions } from '@typescript-eslint/parser';
import { RuleConfig, FlatESLintConfigItem, MergeIntersection, RenamePrefix, TypeScriptRules, VitestRules, NRules, Prefix, ReactHooksRules, ReactRules, JSDocRules, ImportRules, EslintRules, JsoncRules, VueRules, UnicornRules, EslintCommentsRules } from '@antfu/eslint-define-config';
import { RequiredOptions, LiteralUnion, BuiltInParserName } from 'prettier';
import { Linter } from 'eslint';
import { Rules } from 'eslint-plugin-antfu';

type PrettierCustomParser = 'toml'
type PrettierParser = BuiltInParserName | PrettierCustomParser
interface PrettierOptions extends RequiredOptions {
  parser: LiteralUnion<PrettierParser>
}
type PartialPrettierExtendedOptions = Partial<PrettierOptions>

/**
 * Use Prettier to format code.
 *
 */
type PrettierRuleConfig = RuleConfig<PrettierOptions>

/**
 * Use Prettier to format code.
 *
 */
interface PrettierRule {
  /**
   * Use Prettier to format code.
   *
   */
  'prettier/prettier': PrettierRuleConfig
}

/**
 * Option.
 */
interface ClassnamesOrderOption {
  callees?: string[]
  ignoredKeys?: string[]
  config?: string | Record<string, any>
  removeDuplicates?: boolean
  tags?: string[]

  [k: string]: any
}

/**
 * Options.
 */
type ClassnamesOrderOptions = [ClassnamesOrderOption?]

/**
 * Enforce a consistent and logical order of the Tailwind CSS classnames.
 * @see [classnames-order](https://github.com/francoismassart/eslint-plugin-tailwindcss/tree/master/docs/rules/classnames-order.md)
 */
type ClassnamesOrderRuleConfig = RuleConfig<ClassnamesOrderOptions>

/**
 * Enforce a consistent and logical order of the Tailwind CSS classnames.
 * @see [classnames-order](https://github.com/francoismassart/eslint-plugin-tailwindcss/tree/master/docs/rules/classnames-order.md)
 */
interface ClassnamesOrderRule {
  /**
   * Enforce a consistent and logical order of the Tailwind CSS classnames.
   * @see [classnames-order](https://github.com/francoismassart/eslint-plugin-tailwindcss/tree/master/docs/rules/classnames-order.md)
   */
  'tailwindcss/classnames-order': ClassnamesOrderRuleConfig
}

/**
 * Option.
 */
interface EnforcesNegativeArbitraryValuesOption {
  callees?: string[]
  ignoredKeys?: string[]
  config?: string | Record<string, any>
  tags?: string[]

  [k: string]: any
}

/**
 * Options.
 */
type EnforcesNegativeArbitraryValuesOptions = [
  EnforcesNegativeArbitraryValuesOption?,
]

/**
 * Warns about dash prefixed classnames using arbitrary values.
 * @see [enforces-negative-arbitrary-values](https://github.com/francoismassart/eslint-plugin-tailwindcss/tree/master/docs/rules/enforces-negative-arbitrary-values.md)
 */
type EnforcesNegativeArbitraryValuesRuleConfig =
  RuleConfig<EnforcesNegativeArbitraryValuesOptions>

/**
 * Warns about dash prefixed classnames using arbitrary values.
 * @see [enforces-negative-arbitrary-values](https://github.com/francoismassart/eslint-plugin-tailwindcss/tree/master/docs/rules/enforces-negative-arbitrary-values.md)
 */
interface EnforcesNegativeArbitraryValuesRule {
  /**
   * Warns about dash prefixed classnames using arbitrary values.
   * @see [enforces-negative-arbitrary-values](https://github.com/francoismassart/eslint-plugin-tailwindcss/tree/master/docs/rules/enforces-negative-arbitrary-values.md)
   */
  'tailwindcss/enforces-negative-arbitrary-values': EnforcesNegativeArbitraryValuesRuleConfig
}

/**
 * Option.
 */
interface EnforcesShorthandOption {
  callees?: string[]
  ignoredKeys?: string[]
  config?: string | Record<string, any>
  tags?: string[]

  [k: string]: any
}

/**
 * Options.
 */
type EnforcesShorthandOptions = [EnforcesShorthandOption?]

/**
 * Enforces the usage of shorthand Tailwind CSS classnames.
 * @see [enforces-shorthand](https://github.com/francoismassart/eslint-plugin-tailwindcss/tree/master/docs/rules/enforces-shorthand.md)
 */
type EnforcesShorthandRuleConfig = RuleConfig<EnforcesShorthandOptions>

/**
 * Enforces the usage of shorthand Tailwind CSS classnames.
 * @see [enforces-shorthand](https://github.com/francoismassart/eslint-plugin-tailwindcss/tree/master/docs/rules/enforces-shorthand.md)
 */
interface EnforcesShorthandRule {
  /**
   * Enforces the usage of shorthand Tailwind CSS classnames.
   * @see [enforces-shorthand](https://github.com/francoismassart/eslint-plugin-tailwindcss/tree/master/docs/rules/enforces-shorthand.md)
   */
  'tailwindcss/enforces-shorthand': EnforcesShorthandRuleConfig
}

/**
 * Option.
 */
interface MigrationFromTailwind_2Option {
  callees?: string[]
  ignoredKeys?: string[]
  config?: string | Record<string, any>
  tags?: string[]

  [k: string]: any
}

/**
 * Options.
 */
type MigrationFromTailwind_2Options = [MigrationFromTailwind_2Option?]

/**
 * Detect obsolete classnames when upgrading to Tailwind CSS v3.
 * @see [migration-from-tailwind-2](https://github.com/francoismassart/eslint-plugin-tailwindcss/tree/master/docs/rules/migration-from-tailwind-2.md)
 */
type MigrationFromTailwind_2RuleConfig =
  RuleConfig<MigrationFromTailwind_2Options>

/**
 * Detect obsolete classnames when upgrading to Tailwind CSS v3.
 * @see [migration-from-tailwind-2](https://github.com/francoismassart/eslint-plugin-tailwindcss/tree/master/docs/rules/migration-from-tailwind-2.md)
 */
interface MigrationFromTailwind_2Rule {
  /**
   * Detect obsolete classnames when upgrading to Tailwind CSS v3.
   * @see [migration-from-tailwind-2](https://github.com/francoismassart/eslint-plugin-tailwindcss/tree/master/docs/rules/migration-from-tailwind-2.md)
   */
  'tailwindcss/migration-from-tailwind-2': MigrationFromTailwind_2RuleConfig
}

/**
 * Option.
 */
interface NoArbitraryValueOption {
  callees?: string[]
  ignoredKeys?: string[]
  config?: string | Record<string, any>
  tags?: string[]

  [k: string]: any
}

/**
 * Options.
 */
type NoArbitraryValueOptions = [NoArbitraryValueOption?]

/**
 * Forbid using arbitrary values in classnames.
 * @see [no-arbitrary-value](https://github.com/francoismassart/eslint-plugin-tailwindcss/tree/master/docs/rules/no-arbitrary-value.md)
 */
type NoArbitraryValueRuleConfig = RuleConfig<NoArbitraryValueOptions>

/**
 * Forbid using arbitrary values in classnames.
 * @see [no-arbitrary-value](https://github.com/francoismassart/eslint-plugin-tailwindcss/tree/master/docs/rules/no-arbitrary-value.md)
 */
interface NoArbitraryValueRule {
  /**
   * Forbid using arbitrary values in classnames.
   * @see [no-arbitrary-value](https://github.com/francoismassart/eslint-plugin-tailwindcss/tree/master/docs/rules/no-arbitrary-value.md)
   */
  'tailwindcss/no-arbitrary-value': NoArbitraryValueRuleConfig
}

/**
 * Option.
 */
interface NoContradictingClassnameOption {
  callees?: string[]
  ignoredKeys?: string[]
  config?: string | Record<string, any>
  tags?: string[]

  [k: string]: any
}

/**
 * Options.
 */
type NoContradictingClassnameOptions = [NoContradictingClassnameOption?]

/**
 * Avoid contradicting Tailwind CSS classnames (e.g. "w-3 w-5").
 * @see [no-contradicting-classname](https://github.com/francoismassart/eslint-plugin-tailwindcss/tree/master/docs/rules/no-contradicting-classname.md)
 */
type NoContradictingClassnameRuleConfig =
  RuleConfig<NoContradictingClassnameOptions>

/**
 * Avoid contradicting Tailwind CSS classnames (e.g. "w-3 w-5").
 * @see [no-contradicting-classname](https://github.com/francoismassart/eslint-plugin-tailwindcss/tree/master/docs/rules/no-contradicting-classname.md)
 */
interface NoContradictingClassnameRule {
  /**
   * Avoid contradicting Tailwind CSS classnames (e.g. "w-3 w-5").
   * @see [no-contradicting-classname](https://github.com/francoismassart/eslint-plugin-tailwindcss/tree/master/docs/rules/no-contradicting-classname.md)
   */
  'tailwindcss/no-contradicting-classname': NoContradictingClassnameRuleConfig
}

/**
 * Option.
 */
interface NoCustomClassnameOption {
  callees?: string[]
  ignoredKeys?: string[]
  config?: string | Record<string, any>
  cssFiles?: string[]
  cssFilesRefreshRate?: number
  tags?: string[]
  whitelist?: string[]

  [k: string]: any
}

/**
 * Options.
 */
type NoCustomClassnameOptions = [NoCustomClassnameOption?]

/**
 * Detect classnames which do not belong to Tailwind CSS.
 * @see [no-custom-classname](https://github.com/francoismassart/eslint-plugin-tailwindcss/tree/master/docs/rules/no-custom-classname.md)
 */
type NoCustomClassnameRuleConfig = RuleConfig<NoCustomClassnameOptions>

/**
 * Detect classnames which do not belong to Tailwind CSS.
 * @see [no-custom-classname](https://github.com/francoismassart/eslint-plugin-tailwindcss/tree/master/docs/rules/no-custom-classname.md)
 */
interface NoCustomClassnameRule {
  /**
   * Detect classnames which do not belong to Tailwind CSS.
   * @see [no-custom-classname](https://github.com/francoismassart/eslint-plugin-tailwindcss/tree/master/docs/rules/no-custom-classname.md)
   */
  'tailwindcss/no-custom-classname': NoCustomClassnameRuleConfig
}

/**
 * All TailwindCss rules.
 */
type TailwindCssRules = ClassnamesOrderRule &
  EnforcesNegativeArbitraryValuesRule &
  EnforcesShorthandRule &
  MigrationFromTailwind_2Rule &
  NoArbitraryValueRule &
  NoContradictingClassnameRule &
  NoCustomClassnameRule

type WrapRuleConfig<T extends {
    [key: string]: any;
}> = {
    [K in keyof T]: T[K] extends RuleConfig ? T[K] : RuleConfig<T[K]>;
};
type EslintFlatRules = WrapRuleConfig<MergeIntersection<RenamePrefix<TypeScriptRules, '@typescript-eslint/', 'ts/'> & RenamePrefix<VitestRules, 'vitest/', 'test/'> & RenamePrefix<NRules, 'n/', 'node/'> & Prefix<Rules, 'antfu/'> & ReactHooksRules & ReactRules & JSDocRules & ImportRules & EslintRules & JsoncRules & VueRules & UnicornRules & PrettierRule & TailwindCssRules & EslintCommentsRules & {
    'test/no-only-tests': RuleConfig<[]>;
}>>;
type FlatConfigItem = Omit<FlatESLintConfigItem<EslintFlatRules, false>, 'plugins'> & {
    /**
     * Custom name of each config item
     */
    name?: string;
    /**
     * An object containing a name-value mapping of plugin names to plugin objects. When `files` is specified, these plugins are only available to the matching files.
     *
     * @see [Using plugins in your configuration](https://eslint.org/docs/latest/user-guide/configuring/configuration-files-new#using-plugins-in-your-configuration)
     */
    plugins?: Record<string, any>;
};
type UserConfigItem = FlatConfigItem | Linter.FlatConfig;

type Awaitable<T> = T | Promise<T>;
type OptionsTypescript = (OptionsTypeScriptWithTypes & OptionsOverrides) | (OptionsTypeScriptParserOptions & OptionsOverrides);
interface OptionsFormatters {
    /**
     * Enable formatting support for HTML.
     */
    html?: boolean;
    /**
     * Enable formatting support for CSS, Less, Sass, and SCSS.
     */
    css?: boolean;
    /**
     * Enable formatting support for Markdown.
     */
    markdown?: boolean;
    /**
     * Enable formatting support for GraphQL.
     */
    graphql?: boolean;
    /**
     * Enable formatting support for Yaml.
     */
    yaml?: boolean;
    /**
     * Enable formatting support for Toml.
     */
    toml?: boolean;
}
interface OptionsComponentExts {
    /**
     * Additional extensions for components.
     *
     * @example ['vue']
     * @default []
     */
    componentExts?: string[];
}
interface OptionsTypeScriptParserOptions {
    /**
     * Additional parser options for TypeScript.
     */
    parserOptions?: Partial<ParserOptions>;
    /**
     * Glob patterns for files that should be type aware.
     * @default ['**\/*.{ts,tsx}']
     */
    filesTypeAware?: string[];
}
interface OptionsTypeScriptWithTypes {
    /**
     * When this options is provided, type aware rules will be enabled.
     * @see https://typescript-eslint.io/linting/typed-linting/
     */
    tsconfigPath?: string | string[];
}
interface OptionsOverrides {
    overrides?: FlatConfigItem['rules'];
}
interface OptionsUnoCSS extends OptionsOverrides {
    /**
     * Enable attributify support.
     * @default true
     */
    attributify?: boolean;
    /**
     * Enable strict mode by throwing errors about blocklisted classes.
     * @default false
     */
    strict?: boolean;
}
interface OptionsVue extends OptionsOverrides {
    /**
     * The vue version
     *
     * @default 3
     */
    version?: 2 | 3;
}
interface OptionsConfig extends OptionsComponentExts {
    /**
     * The current working directory
     *
     * @default process.cwd()
     */
    cwd?: string;
    /**
     * Enable gitignore support.
     *
     * Passing an object to configure the options.
     *
     * @see https://github.com/antfu/eslint-config-flat-gitignore
     * @default true
     */
    gitignore?: boolean | FlatGitignoreOptions;
    /**
     * Core rules. Can't be disabled.
     */
    javascript?: OptionsOverrides;
    /**
     * Enable TypeScript support.
     *
     * Passing an object to enable TypeScript Language Server support.
     *
     * @default auto-detect based on the dependencies
     */
    typescript?: boolean | OptionsTypescript;
    /**
     * Enable test support.
     *
     * @default true
     */
    test?: boolean | OptionsOverrides;
    /**
     * Enable Vue support.
     *
     * @default auto-detect based on the dependencies
     */
    vue?: boolean | OptionsVue;
    /**
     * Enable JSONC support.
     *
     * @default true
     */
    jsonc?: boolean | OptionsOverrides;
    /**
     * Enable react rules.
     *
     * Requires installing:
     * - `eslint-plugin-react`
     * - `eslint-plugin-react-hooks`
     * - `eslint-plugin-react-refresh`
     *
     * @default false
     */
    react?: boolean | OptionsOverrides;
    /**
     * Enable svelte rules.
     *
     * Requires installing:
     * - `eslint-plugin-svelte`
     *
     * @default false
     */
    svelte?: boolean | OptionsOverrides;
    /**
     * Enable tainwindcss rules.
     *
     * @default auto-detect based on the dependencies
     */
    tailwindcss?: boolean | OptionsOverrides;
    /**
     * Enable unocss rules.
     *
     * Requires installing:
     * - `@unocss/eslint-plugin`
     *
     * @default false
     */
    unocss?: boolean | OptionsUnoCSS;
    /**
     * Whether to use prettierrc
     *
     * If true, the rules in prettierrc will override the default rules
     *
     * @default true
     */
    usePrettierrc?: boolean;
    /**
     * Use external formatters to format files.
     *
     * Requires installing:
     * - `eslint-plugin-prettier`
     *
     * @default
     * {
     *  "html": true,
     *  "css": true,
     *  "graphql": false,
     *  "markdown": false
     *  "yaml": false
     *  "toml": false
     * }
     */
    formatter?: OptionsFormatters;
    /**
     * Default prettier rules
     *
     * @default
     * ```json
     * {
     *   "arrowParens": "avoid",
     *   "htmlWhitespaceSensitivity": "ignore"
     *   "printWidth": 80,
     *   "semi": false,
     *   "singleQuote": true,
     * }
     * ```
     */
    prettierRules?: PartialPrettierExtendedOptions;
    /**
     * Control to disable some rules in editors.
     * @default auto-detect based on the process.env
     */
    isInEditor?: boolean;
}

/**
 * Construct an array of ESLint flat config items.
 */
declare function defineConfig(options?: OptionsConfig & FlatConfigItem, ...userConfigs: Awaitable<UserConfigItem | UserConfigItem[]>[]): Promise<UserConfigItem[]>;

export { defineConfig };
